﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace SimpleGraphic
{
    public partial class Form1 : Form
    {
        float M = 100; // M пикселей - одна единица
        float D = 1.0f;   // Шаг меток
        float PHI = 4 * (float) Math.PI;
        float Delta = 0.001f; // В радианах
        Point[] P;
        Button bd, bp;

        // Функции масштабирования
        private float i2x(int i) {  return i / M;  }
        private float j2y(int j) { return - j / M; }
        private int x2i(float x) { return (int) Math.Round(x * M); }
        private int y2j(float y) { return -(int)Math.Round(y * M); }

        // Перевод из полярных в декартовы координаты
        private void decart(float phi, float rho, out float x, out float y)
        {
            x = rho * (float) Math.Cos(phi);
            y = rho * (float) Math.Sin(phi);
        }
        
        public Form1()
        {
            InitializeComponent();
            SetStyle(ControlStyles.ResizeRedraw, true);
        }

        protected override void OnMouseWheel(MouseEventArgs e)
        {
            base.OnMouseWheel(e);
            if (e.Delta > 0) M += 2;
            else if (M > 10) M -= 2;
            OnSizeChanged(null);
        }

        protected override void OnSizeChanged(EventArgs e)
        {
            base.OnSizeChanged(e);
            if (bd != null) DecartButton_Click(bd, null);
            else if (bp != null) PolarButton_Click(bp, null);
            else Invalidate();
        }

        private void DrawSystem(Graphics g)
        {
            float d = D; // Переменная для разметки меток
            int   k = 0; // Экранная координата метки
            string s = ""; // Для подписей
            Font f = new Font("Arial",10);

            // Горизонтальная ось
            g.DrawLine(Pens.Green,
                -ClientRectangle.Width / 2, 0,
                +ClientRectangle.Width / 2, 0);

            do
            {   k = x2i(d); g.DrawLine(Pens.Green, +k, 0, +k, 5);
                            g.DrawLine(Pens.Green, -k, 0, -k, 5);
                s = d.ToString();
                g.DrawString(s, f, Brushes.DarkGreen, k-5, 6);
                g.DrawString("-"+s, f, Brushes.DarkGreen, -k - 10, 6);
                d += D;
            } while (k < ClientRectangle.Width / 2);

            // Вертикальная ось
            g.DrawLine(Pens.Green, 0,  ClientRectangle.Height/2,
                                   0, -ClientRectangle.Height/2);

            d = D;
            do
            {   k = y2j(d); g.DrawLine(Pens.Green, 0, k, 5, k);
                g.DrawLine(Pens.Green, 0, -k, 5, -k);
                s = d.ToString();
                g.DrawString(s, f, Brushes.DarkGreen, 6, k-8);
                g.DrawString("-" + s, f, Brushes.DarkGreen, 6, -k-8);
                d += D;
            } while (k > -ClientRectangle.Height / 2);
        } 

        private void Form1_Paint(object sender, PaintEventArgs e)
        {
            Graphics g = e.Graphics;
            g.TranslateTransform(ClientRectangle.Width / 2, ClientRectangle.Height / 2);
            DrawSystem(g);
            if (P != null) g.DrawLines(Pens.Red, P);
        }

        private void DecartButton_Click(object sender, EventArgs e)
        {
            bd = (Button)sender; bp = null;
            P = new Point[2*(ClientRectangle.Width/2)];
            for (int i = -ClientRectangle.Width / 2; i < ClientRectangle.Width / 2; i++)
            {
                float x = i2x(i);
                float y;
                switch (bd.Text)
                {
                    case "Parabola": y = x * x;
                        break;
                    case "Sinus": y = (float) Math.Sin(x);
                        break;
                    case "Sinus 1/x": 
                        if (x != 0) y = (float)Math.Sin(1 / x);
                        else y = 0;
                        break;
                    default: y = 0;
                        break;
                }
                int j = y2j(y);
                P[i + ClientRectangle.Width / 2].X = i;
                P[i + ClientRectangle.Width / 2].Y = j;
            }
            Invalidate();
        }

        private void PolarButton_Click(object sender, EventArgs e)
        {
            bp = (Button)sender; bd = null;
            float phi, rho;
            float x, y;
            int N = (int) (PHI / Delta);
            P = new Point[N];
            for (int i = 0; i < N; i++ )
            {
                phi = i * Delta;
                switch (bp.Text)
                {
                    case "Spiral": rho = 0.2f*phi;
                        break;
                    case "Rose": rho = 2*(float)Math.Cos(7 * phi);
                        break;
                    case "Cardioida": rho = 1+ (float)Math.Cos(phi);
                        break;
                    default: rho = 0;
                        break;
                }
                decart(phi, rho, out x, out y);
                P[i].X = x2i(x);
                P[i].Y = y2j(y);
            }
            Invalidate();
        }
    }
}
